
global_var	experimental_scripts, report_verbosity, log_verbosity, debug_level, thorough_tests;
experimental_scripts = 0;
report_verbosity = 0;
debug_level = 0;
log_verbosity = 0;
thorough_tests = 0;

{
local_var __gs_opt;
__gs_opt = get_kb_item("global_settings/report_verbosity");
if (__gs_opt)
  if ("Verbose" >< __gs_opt) report_verbosity = 2;
  else if ("Normal" >< __gs_opt) report_verbosity = 1;
  else if ("Quiet" >< __gs_opt) report_verbosity = 0;

__gs_opt = get_kb_item("global_settings/log_verbosity");
if (__gs_opt)
  if ("Verbose" >< __gs_opt) log_verbosity = 2;
  else if ("Normal" >< __gs_opt) log_verbosity = 1;
  else if ("Quiet" >< __gs_opt) log_verbosity = 0;

__gs_opt = get_kb_item("global_settings/debug_level");
if (__gs_opt =~ '^[0-9]+$') debug_level = int(__gs_opt);


__gs_opt = get_kb_item("global_settings/experimental_scripts");
if ("yes" >< __gs_opt) experimental_scripts = 1;

__gs_opt = get_kb_item("global_settings/thorough_tests");
if ("yes" >< __gs_opt) thorough_tests = 1;
}
